#!/usr/bin/python
#  
#  vim: expandtab:tw=68:ts=4:sw=4:
# 
#  (c) Copyright 2008 Samsung Telecommunications America, Inc.
#                   All rights reserved
# 
#              Wireless Terminals Lab R & D
# 
# 
#  File: install_gtk.py
#  Author: Mohan Reddy Duggi (mreddy@sta.samsung.com)
#  Creation date: Wed May 21 13:37:07 2008
#  Rev: $Id$
# 

import sys;
import os;
import getopt;
import string;
import urllib;
from posixpath import exists;

from config import Pkg_info;
from config import Env_kvp;

def usage(prog_name) :
    usage_str = """
$prog_name: A script to download, patch, build and install gtk and its dependencies automatically.

Usage: $prog_name [Options]

Options:
  --help        Show this help message and quit.
  --install-prefix=p    Path where the gtk libs should be installed.
                        Default value: /usr/work/dfb_arm
  --src-prefix=p    Path where the pre-downloaded gtk source directories exist.
                    Default value: ./
  --patch-prefix=p  Path where the patches exist
  --only=pkg        Build only a single package
  --start-with-pkg=name  Start building with the given package
  --clean           Execute "make distclean" before building the package
  --fresh-build     Remove the un-packed directories and start fresh


Example:

1) Download, patch, build and install packages (Generally for the first time)

   $prog_name --install-prefix=/usr/work/dfb_arm --src-prefix=/usr/work/gtk --patch-prefix=/root/patches

2) Build & install packages without patching

   $prog_name --install-prefix=/usr/work/dfb_arm --src-prefix=/usr/work/gtk

3) Start build from specified package

   $prog_name --install-prefix=/usr/work/dfb --src-prefix=/root/packages --clean

4) Remove already unpacked packages and do a fresh build

   $prog_name --install-prefix=/usr/work/dfb_arm --src-prefix=/usr/work/gtk --patch-prefix=/root/patches --fresh-build
   
5) Clean build and install packages

   $prog_name --install-prefix=/usr/work/dfb --src-prefix=/usr/work/gtk --clean

""";

    prog_name = prog_name[prog_name.rfind ("/") + 1:];
    str_list = usage_str.split ("$prog_name");

    print prog_name.join (str_list);


def exec_shell_cmd (cmd) :
    print "Executing: " + cmd + "\n";

    if os.system (cmd) != 0 :
        print "Command " + cmd + " failed\n";
        sys.exit(3);


def make_abs_prefix (prefix) :
    if prefix[0] != "/" :
        prefix = os.getcwd () + "/" + prefix;

    return prefix;


def parse_args (prog_name, args, conf) :
    try :
        opts, args = getopt.getopt (args, "", ["help", "install-prefix=", "src-prefix=", "patch-prefix=", "start-with-pkg=", "fresh-build", "clean", "only="]);
    except getopt.GetoptError:
        print "Invalid arguments passed. Try " +  prog_name + " --help";
        usage (prog_name);
        sys.exit(2);

    conf["install-prefix"] = "/usr/work/dfb_arm";
    conf["src-prefix"] = os.getcwd ();
    #conf["patch-prefix"] = conf["src-prefix"] + "./platform_patches";
    conf["patch-prefix"] = "";
    conf["start-with-pkg"] = "";
    conf["fresh-build"] = 0;
    conf["clean"] = 0;
    conf["only"] = "";


    for opt, arg in opts :
        if opt == "--help" :
            usage (prog_name);
            sys.exit(2);
        elif opt == "--install-prefix" :
            conf["install-prefix"] = make_abs_prefix (arg);
        elif opt == "--src-prefix" :
            conf["src-prefix"] = make_abs_prefix (arg);
        elif opt == "--patch-prefix" :
            conf["patch-prefix"] = make_abs_prefix (arg);
        elif opt == "--start-with-pkg" :
            conf["start-with-pkg"] = arg;
        elif opt == "--fresh-build" :
            conf["fresh-build"] = 1;
        elif opt == "--clean" :
            conf["clean"] = 1;
        elif opt == "--only" :
            conf["only"] = arg;

    if conf["install-prefix"] == "":
        print "\n--install-prefix option is necessary\n";
        sys.exit(2);

    if conf["start-with-pkg"] != "" and conf["only"] != "" :
        print "\n--start-with-pkg and --only options shouldn't be given same time\n";
        sys.exit(2);


def dfb_env_setup (prefix) :
    old_dir = os.getcwd();
    print "install-prefix = " + prefix + "\n";
    if os.path.exists (prefix) == 0 :
        os.mkdir (prefix);
    os.chdir (prefix);
    exec_shell_cmd ('mkdir -p man/man1 bin lib etc include info sbin share src');
    os.chdir (old_dir);

    for key in Env_kvp.keys () :
        vals = Env_kvp[key].split ("$install_prefix");

        if len (vals) == 2 :
            vals = vals[0] + prefix + vals[1];
        else :
            vals = vals[0];
    
        if os.environ.has_key (key) :
            if key.find ("PATH") < 0 :
                os.environ[key] =  vals + " " + os.environ[key];
            else :
                os.environ[key] =  vals + ":" + os.environ[key];
        else :
            os.environ[key] = vals;

        print "export " + key + "=" + os.environ[key] + "\n";


def download_and_unpack_pkg (pkg, conf) :

    #If the package directory already exists, don't download
    if exists (conf["src-prefix"] + "/" + pkg["name"]) :
        return;

    url = pkg["download"];
    file = url[url.rfind ("/") + 1:];

    #If the package doesn't exist, download it.
    if exists (file) == 0 :
        print "Downloading " + url + " to " + file;
        urllib.urlretrieve (url, file);

    type = file[file.rfind (".") + 1:];

    if type == "bz2" :
        cmd = "tar -xjvf " + file;
    elif type == "gz" :
        cmd = "tar -xzvf " + file;

    exec_shell_cmd (cmd);


def patch_pkg (pkg, conf) :
    for patch in pkg["patches"] :
        patch_cmd = "patch -p1 < " + conf["patch-prefix"] + "/" + patch;
        exec_shell_cmd (patch_cmd);


def build_pkg (pkg, conf) :

    conf_opts = "";
    if pkg.has_key ("config-opts") :
        conf_opts = pkg["config-opts"];

    config_cmd = "./configure " + "--prefix=" + conf["install-prefix"] + " " + conf_opts;

    if conf["clean"] :
        try :
            exec_shell_cmd ("make distclean");
        except :
            print "Ignoring make distclean failure..\n";

    print config_cmd + "\n";

    exec_shell_cmd (config_cmd);

    if pkg.has_key ("make") :
        exec_shell_cmd (pkg["make"]);
    else :
        exec_shell_cmd ("make");


def install_pkg (pkg) :
    exec_shell_cmd ("make install");


if __name__ == "__main__" :
    Conf = {};
    start_flag = 1;
    parse_args (sys.argv[0], sys.argv[1:], Conf);

    dfb_env_setup (Conf["install-prefix"]);

    if Conf["start-with-pkg"] != "" :
        start_flag = 0;

    for pkg in Pkg_info :

        if start_flag == 0 and pkg["name"] == Conf["start-with-pkg"] :
            start_flag = 1;

        if start_flag == 1 :
            if Conf["only"] != "" and pkg["name"] != Conf["only"] :
                continue;

            os.chdir (Conf["src-prefix"]);

            if Conf["fresh-build"] == 1 :
                exec_shell_cmd ("rm -rf " + pkg["name"]);

            download_and_unpack_pkg (pkg, Conf);
            os.chdir (Conf["src-prefix"] + "/"+ pkg["name"]);

            if Conf["patch-prefix"] != "" and pkg.has_key ("patches") :
                patch_pkg (pkg, Conf);

            build_pkg (pkg, Conf);
            install_pkg (pkg);

# EOF

